import { auth } from './firebase';
import { db } from './firebase';
import { 
  signInWithEmailAndPassword,
  signOut as firebaseSignOut,
  onAuthStateChanged,
  User,
  getIdToken,
  createUserWithEmailAndPassword
} from 'firebase/auth';
import { doc, getDoc } from 'firebase/firestore';
import { AdminUser } from '@/types/education';
import { initializeAdminUser } from './admin_setup';

const ADMIN_EMAILS = [
  'akalagborojohn@gmail.com',
  'edvolabnomvate@gmail.com'
];

// Maximum retries for operations
const MAX_RETRIES = 3;
const RETRY_DELAY = 1000; // 1 second

// Helper function to wait
const wait = (ms: number) => new Promise(resolve => setTimeout(resolve, ms));

// Helper function to check network status
const isOnline = () => typeof navigator !== 'undefined' && navigator.onLine;

// Helper function for retrying operations
async function retry<T>(
  operation: () => Promise<T>,
  retries = MAX_RETRIES,
  delay = RETRY_DELAY
): Promise<T> {
  try {
    return await operation();
  } catch (error: any) {
    if (retries > 0 && error.message?.includes('offline')) {
      console.log(`Retrying operation. Attempts remaining: ${retries}`);
      await wait(delay);
      return retry(operation, retries - 1, delay * 2);
    }
    throw error;
  }
}

// Helper function to set cookies
function setCookie(name: string, value: string, maxAge: number = 3600) {
  console.log(`Setting cookie: ${name}`);
  document.cookie = `${name}=${value}; path=/; max-age=${maxAge}; secure; samesite=strict`;
}

// Helper function to remove cookies
function removeCookie(name: string) {
  console.log(`Removing cookie: ${name}`);
  document.cookie = `${name}=; path=/; expires=Thu, 01 Jan 1970 00:00:00 GMT`;
}

// Check if email is allowed for admin access
export function isAllowedAdminEmail(email: string): boolean {
  console.log(`Checking admin email: ${email}`);
  const isAllowed = ADMIN_EMAILS.includes(email);
  console.log(`Is admin email allowed: ${isAllowed}`);
  return isAllowed;
}

// Check if a user is an admin with retry logic
export async function isAdmin(user: User | null): Promise<boolean> {
  console.log('Checking admin status for user:', user?.email);
  if (!user) {
    console.log('No user provided');
    return false;
  }
  
  try {
    // First check if the email is in the allowed list
    if (!isAllowedAdminEmail(user.email || '')) {
      console.log('Email not in allowed list');
      removeCookie('admin_status');
      return false;
    }

    try {
      const adminDoc = await retry(() => getDoc(doc(db, 'adminUsers', user.uid)));
      console.log('Admin doc exists:', adminDoc.exists());
      
      // If user is allowed but not initialized, initialize them
      if (!adminDoc.exists() && user.email) {
        console.log('Initializing new admin user');
        await retry(() => initializeAdminUser(user.uid, user.email!));
        setCookie('admin_status', 'true');
        return true;
      }
      
      if (adminDoc.exists()) {
        setCookie('admin_status', 'true');
        return true;
      }
      
      removeCookie('admin_status');
      return false;
    } catch (firestoreError) {
      console.error('Firestore error, falling back to email check:', firestoreError);
      // If Firestore fails, fall back to email check
      const isAllowed = isAllowedAdminEmail(user.email || '');
      if (isAllowed) {
        setCookie('admin_status', 'true');
      } else {
        removeCookie('admin_status');
      }
      return isAllowed;
    }
  } catch (error) {
    console.error('Error checking admin status:', error);
    removeCookie('admin_status');
    return false;
  }
}

// Get admin user data with retry and offline handling
export async function getAdminData(user: User | null): Promise<AdminUser | null> {
  if (!user) return null;
  
  try {
    // First check if the email is in the allowed list
    if (!isAllowedAdminEmail(user.email || '')) {
      return null;
    }

    // Try to get data from cache first
    const cachedData = getCachedAdminData();
    if (!isOnline() && cachedData) {
      console.log('Using cached admin data (offline)');
      return cachedData;
    }

    // Attempt to get fresh data with retries
    const adminDoc = await retry(() => getDoc(doc(db, 'adminUsers', user.uid)));
    
    // If offline and document doesn't exist, use fallback data
    if (!adminDoc.exists() && !isOnline()) {
      console.log('Offline mode: Using fallback admin data');
      const fallbackData: AdminUser = {
        id: user.uid,
        email: user.email || '',
        role: 'ADMIN',
        permissions: ['GAMES', 'USERS', 'CONTENT'],
        createdAt: new Date(),
        updatedAt: new Date(),
        isActive: true
      };
      cacheAdminData(fallbackData);
      return fallbackData;
    }

    // If user is allowed but not initialized, initialize them
    if (!adminDoc.exists() && user.email) {
      console.log('Initializing new admin user');
      await retry(() => initializeAdminUser(user.uid, user.email!));
      return getAdminData(user); // Retry after initialization
    }

    if (adminDoc.exists()) {
      const data = adminDoc.data() as AdminUser;
      cacheAdminData(data);
      return data;
    }

    return null;
  } catch (error) {
    console.error('Error fetching admin data:', error);
    
    // Try to use cached data as fallback
    const cachedData = getCachedAdminData();
    if (cachedData) {
      console.log('Using cached admin data after error');
      return cachedData;
    }
    
    return null;
  }
}

// Cache admin data in localStorage
function cacheAdminData(data: AdminUser | null) {
  if (typeof window === 'undefined') return;
  
  if (data) {
    localStorage.setItem('adminData', JSON.stringify({
      data,
      timestamp: Date.now()
    }));
  } else {
    localStorage.removeItem('adminData');
  }
}

// Get cached admin data
function getCachedAdminData(): AdminUser | null {
  if (typeof window === 'undefined') return null;
  
  const cached = localStorage.getItem('adminData');
  if (!cached) return null;

  try {
    const { data, timestamp } = JSON.parse(cached);
    // Cache expires after 5 minutes
    if (Date.now() - timestamp > 5 * 60 * 1000) {
      localStorage.removeItem('adminData');
      return null;
    }
    return data;
  } catch {
    localStorage.removeItem('adminData');
    return null;
  }
}

// Admin sign in
export async function adminSignIn(
  email: string,
  password: string,
  redirectPath?: string
): Promise<{
  success: boolean;
  error?: string;
  isAdmin?: boolean;
  redirectTo?: string;
}> {
  console.log('Starting admin sign in for:', email);
  try {
    const userCredential = await signInWithEmailAndPassword(auth, email, password);
    console.log('User signed in successfully');
    
    // Check if user is an allowed admin
    if (isAllowedAdminEmail(userCredential.user.email || '')) {
      console.log('Email is in allowed admin list');
      const adminCheck = await isAdmin(userCredential.user);
      console.log('Admin check result:', adminCheck);
      
      if (adminCheck) {
        // Set admin cookies and redirect to admin page
        const token = await getIdToken(userCredential.user);
        console.log('Setting admin cookies');
        setCookie('session', token);
        setCookie('admin_status', 'true');
        
        return {
          success: true,
          isAdmin: true,
          redirectTo: redirectPath || '/admin'
        };
      }
    }

    // For non-admin users, just set the session token
    console.log('Setting regular user session');
    const token = await getIdToken(userCredential.user);
    setCookie('session', token);
    
    return {
      success: true,
      isAdmin: false,
      redirectTo: '/games'  // Redirect to games page for regular users
    };
  } catch (error: any) {
    console.error('Sign in error:', error);
    return {
      success: false,
      error: error.message
    };
  }
}

// Admin sign up (only for allowed emails)
export async function adminSignUp(
  email: string,
  password: string
): Promise<{
  success: boolean;
  error?: string;
}> {
  try {
    // First check if the email is in the allowed list
    if (!isAllowedAdminEmail(email)) {
      return {
        success: false,
        error: 'Email not authorized for admin access'
      };
    }

    const userCredential = await createUserWithEmailAndPassword(auth, email, password);
    await initializeAdminUser(userCredential.user.uid, email);
    
    // Set admin status cookie after successful signup
    setCookie('admin_status', 'true');
    
    return {
      success: true
    };
  } catch (error: any) {
    return {
      success: false,
      error: error.message
    };
  }
}

// Admin sign out
export async function adminSignOut(): Promise<void> {
  try {
    // Clear all admin cookies
    removeCookie('session');
    removeCookie('admin_status');
    await firebaseSignOut(auth);
  } catch (error) {
    console.error('Error signing out:', error);
    throw error;
  }
}

// Subscribe to admin auth state changes
export function onAdminAuthStateChanged(
  callback: (user: User | null, isAdmin: boolean) => void
): () => void {
  return onAuthStateChanged(auth, async (user) => {
    console.log('Auth state changed. User:', user?.email);
    if (user) {
      try {
        // Only proceed with token refresh if user is an allowed admin
        if (isAllowedAdminEmail(user.email || '')) {
          console.log('Refreshing admin token');
          const token = await getIdToken(user, true);
          setCookie('session', token);
          const adminCheck = await isAdmin(user);
          console.log('Admin check result:', adminCheck);
          if (adminCheck) {
            setCookie('admin_status', 'true');
          } else {
            removeCookie('admin_status');
          }
        }
      } catch (error) {
        console.error('Error refreshing token:', error);
      }
    } else {
      // Clear cookies when user signs out
      console.log('Clearing cookies on sign out');
      removeCookie('session');
      removeCookie('admin_status');
    }
    const adminStatus = await isAdmin(user);
    callback(user, adminStatus);
  });
}

// Check if user has specific admin permission
export async function hasAdminPermission(
  user: User | null,
  permission: string
): Promise<boolean> {
  if (!user) return false;
  
  try {
    const adminData = await getAdminData(user);
    if (!adminData) return false;
    
    return adminData.permissions.includes(permission);
  } catch (error) {
    console.error('Error checking admin permission:', error);
    return false;
  }
}

export const handleError = (error: Error | unknown): string => {
  if (error instanceof Error) {
    return error.message;
  }
  return 'An unknown error occurred';
};

export const handleAuthError = (error: Error | unknown): string => {
  if (error instanceof Error) {
    return error.message;
  }
  return 'An authentication error occurred';
}; 